@@ -4,17 +4,15 @@
 //!## Example
 //!Basic GET request
 //!```
-//!extern crate http_req;
 //!use http_req::request;
 //!
 //!fn main() {
-//!    let res = request::get("https://doc.rust-lang.org/").unwrap();
+//!    let mut buff = Vec::new();
+//!    let res = request::get("https://doc.rust-lang.org/", &mut buff).unwrap();
 //!
 //!    println!("Status: {} {}", res.status_code(), res.reason());
 //!}
 //!```
-extern crate native_tls;
-
 pub mod error;
 pub mod request;
 pub mod response;
